from datetime import datetime


class VendorDocumentsReport:
    """Report Vendor Documents"""

    def __init__(self, db, company_id: str) -> None:
        self.db = db
        self.company_id = company_id

    def key_func(self, k: dict) -> str:
        """Key function"""
        return k['Vendor Name']

    def match_expiry_date(self, vendor_doc: dict, vendor_qoc_questions: list) -> str:
        """Match expiry date"""
        expiry_date = ''

        if 'values' in vendor_doc['ocrOutput']:

            if vendor_doc['ocrOutput']['values'] != []:

                for item in vendor_doc['ocrOutput']['values']:

                    if item['key'] == 'expiry_date':

                        if item['updatedValue'] != '':
                            expiry_date = item['updatedValue']
                        else:
                            expiry_date = item['ocrValue']
            else:

                for vendor_qoc_question in vendor_qoc_questions:
                    if vendor_doc['id'] == vendor_qoc_question['vendorDocId']:
                        expiry_date = vendor_qoc_question['answer']

        else:

            for vendor_qoc_question in vendor_qoc_questions:
                if vendor_doc['id'] == vendor_qoc_question['vendorDocId']:
                    expiry_date = vendor_qoc_question['answer']

        return expiry_date

    def build_vendor_document_report(self) -> dict:
        """Build vendor document report"""

        headers = ["Vendor Name", "Document Type", "Document Name",
                   "Status", "Expiry Date", "Last Modified"]

        vendor_list_by_employer = self.db.get_results(
            query=f"SELECT c.vendorId, c.vendorName FROM c WHERE c.type = 'vendorList' AND c.employerId = '{self.company_id}'")

        results = [x['vendorId'] for x in vendor_list_by_employer]

        docs = self.db.get_results(
            query=f"SELECT c.id, c.vendorId, c.employerId, c.docType, c.docName, c.status, c.ocrOutput, c._ts FROM c WHERE c.type = 'vendorDoc' AND ARRAY_CONTAINS({results}, c.vendorId) AND c.status != 'Not Submitted'")

        vendor_doc_questions = self.db.get_results(
            query="SELECT * FROM c WHERE c.type = 'vendorDocQuestion' AND c.key = 'expiry_date'")

        results = []
        for doc in docs:

            if 'docName' in doc:
                doc_name = doc['docName']
            else:
                doc_name = ''

            expiry_date = self.match_expiry_date(doc, vendor_doc_questions)

            if 'employerId' in doc:
                if doc['employerId'] == self.company_id:

                    results.append({
                        'Vendor Name': [x['vendorName'] for x in vendor_list_by_employer if x['vendorId'] == doc['vendorId']][0],
                        'Document Type': doc['docType'],
                        'Document Name': doc_name,
                        'Status': doc['status'],
                        'Expiry Date': expiry_date,
                        'Last Modified': datetime.fromtimestamp(doc["_ts"]).strftime("%Y-%m-%d")
                    })
                else:
                    pass

            else:
                results.append({
                    'Vendor Name': [x['vendorName'] for x in vendor_list_by_employer if x['vendorId'] == doc['vendorId']][0],
                    'Document Type': doc['docType'],
                    'Document Name': doc_name,
                    'Status': doc['status'],
                    'Expiry Date': expiry_date,
                    'Last Modified': datetime.fromtimestamp(doc["_ts"]).strftime("%Y-%m-%d")
                })

        rows = sorted(results, key=self.key_func)

        return {
            'file_name': 'Vendor_Documents_Report.csv',
            'headers': headers,
            'rows': rows
        }



from datetime import datetime


class VendorDeficienciesReport:
    """Vendor Deficiencies Report"""

    def __init__(self, db, company_id: str) -> None:
        self.db = db
        self.company_id = company_id

    def get_ratings_updated(self, vendor_list: dict) -> dict:
        """Get ratings updated"""
        if ('ratingsUpdated' in vendor_list) and (vendor_list['ratingsUpdated'] != ''):

            return {
                'dateRatingApplied': datetime.fromtimestamp(
                    int(vendor_list['ratingsUpdated'])).strftime('%Y-%m-%d'),
                'daysSinceRatingApplied': (
                    datetime.now() - datetime.fromtimestamp(int(vendor_list['ratingsUpdated']))).days
            }
        else:
            return {
                'dateRatingApplied': '',
                'daysSinceRatingApplied': ''
            }

    def split_vendor_list_by_deficiency(self, vendor_list: dict) -> list:
        """Split vendor list by deficiency"""
        new_list = []

        try:
            tags = [x['text'] for x in vendor_list['tags']]
        except KeyError:
            tags = []

        if ('vendorOverallItems' in vendor_list) and (len(vendor_list['vendorOverallItems']) > 1):
            for deficiency in vendor_list['vendorOverallItems']:
                new_list.append({
                    'vendor_id': vendor_list['vendorId'],
                    'vendorName': vendor_list['vendorName'],
                    'vendorSubscriptionType': vendor_list['vendorSubscriptionType'],
                    'tags': tags,
                    'vendorOverallRating': vendor_list['vendorOverallRating'],
                    'vendorDeficiency': deficiency,
                    'ratingsUpdated': vendor_list['ratingsUpdated'],
                    'vendorAccountAdmin': vendor_list['vendorAccountAdmin']
                })

        else:
            new_list.append({
                'vendor_id': vendor_list['vendorId'],
                'vendorName': vendor_list['vendorName'],
                'vendorSubscriptionType': vendor_list['vendorSubscriptionType'],
                'tags': tags,
                'vendorOverallRating': vendor_list['vendorOverallRating'],
                'vendorDeficiency': [x for x in vendor_list['vendorOverallItems']][0] if len(vendor_list['vendorOverallItems']) > 0 else '',
                'ratingsUpdated': vendor_list['ratingsUpdated'],
                'vendorAccountAdmin': vendor_list['vendorAccountAdmin']
            })

        return new_list

    def build_vendor_deficiencies_report(self) -> dict:
        """Build vendor deficiencies report"""
        vendor_lists = self.db.get_results(
            query=f"SELECT c.vendorId, c.vendorName, c.vendorSubscriptionType, c.tags, c.vendorOverallRating, c.vendorOverallItems, c.employerDates.ratingsUpdated,\
                c.vendorAccountAdmin from c where c.type = 'vendorList'AND c.employerId = '{self.company_id}'")

        formatted_list = []
        for vendor_list in vendor_lists:
            vendor_list['ratingsUpdated'] = self.get_ratings_updated(
                vendor_list)

            vendor_list: list = self.split_vendor_list_by_deficiency(
                vendor_list)

            for item in vendor_list:
                item['tags'] = '\n'.join(item['tags'])

                formatted_list.append({
                    'vendor_id': item['vendor_id'],
                    'Vendor Name': item['vendorName'],
                    'Subscription Type': item['vendorSubscriptionType'],
                    'Vendor Tags': item['tags'],
                    'Vendor Rating': item['vendorOverallRating'],
                    'Current Deficiencies': item['vendorDeficiency'],
                    'Date Rating Applied': item['ratingsUpdated']['dateRatingApplied'],
                    'Days Since Rating Applied': item['ratingsUpdated']['daysSinceRatingApplied'],
                    'Admin Name': item['vendorAccountAdmin']['name'],
                    'Admin Email': item['vendorAccountAdmin']['email'],
                    'Admin Phone': item['vendorAccountAdmin']['phone']
                })

        return {
            'file_name': 'Def_Report.csv',
            'headers': list(formatted_list[0].keys()),
            'rows': formatted_list
        }
